/* TO DO LIST:
- For both "MCQ" and "FIB", maybe add weightings to the questions? Otherwise a new attribute under tests?
*/

-- TEACHER table
CREATE TABLE teachers (
    id INTEGER PRIMARY KEY AUTOINCREMENT,
    first_name TEXT NOT NULL,
    last_name TEXT NOT NULL,
    title TEXT,
    username TEXT UNIQUE NOT NULL,
    password TEXT NOT NULL
);

-- MODULE table
CREATE TABLE modules (
    id INTEGER PRIMARY KEY AUTOINCREMENT,
    code TEXT NOT NULL UNIQUE,
    title TEXT NOT NULL
);

-- STUDENT table
CREATE TABLE students (
    id INTEGER PRIMARY KEY AUTOINCREMENT,
    first_name TEXT NOT NULL,
    last_name TEXT NOT NULL,
    username TEXT UNIQUE NOT NULL,
    password TEXT NOT NULL
);

-- MCQ table (Multiple Choice Questions)
CREATE TABLE MCQ (
    id INTEGER PRIMARY KEY AUTOINCREMENT,
    teacher_id INTEGER NOT NULL,
    module_id INTEGER NOT NULL,
    question TEXT NOT NULL,
    option_A TEXT NOT NULL,
    option_B TEXT NOT NULL,
    option_C TEXT NOT NULL,
    option_D TEXT NOT NULL,
    answer TEXT CHECK (answer IN ('A', 'B', 'C', 'D')) NOT NULL,
    difficulty TEXT CHECK (difficulty IN ('Easy', 'Intermediate', 'Hard')) NOT NULL,
    learning_obj TEXT CHECK (learning_obj IN ('Remember', 'Understand', 'Apply', 'Analyse', 'Evaluate', 'Create')),
    feedback,
    active BOOLEAN DEFAULT TRUE
);

-- Fill in the Blanks table
CREATE TABLE FIB (
    id INTEGER PRIMARY KEY AUTOINCREMENT,
    teacher_id INTEGER NOT NULL,
    module_id INTEGER NOT NULL,
    question TEXT NOT NULL,
    answer TEXT NOT NULL,
    difficulty TEXT CHECK (difficulty IN ('Easy', 'Intermediate', 'Hard')) NOT NULL,
    learning_obj TEXT CHECK (learning_obj IN ('Remember', 'Understand', 'Apply', 'Analyse', 'Evaluate', 'Create')),
    feedback TEXT,
    active BOOLEAN DEFAULT TRUE
);

-- TEST table
CREATE TABLE tests (
    id INTEGER PRIMARY KEY AUTOINCREMENT,
    teacher_id INTEGER NOT NULL,
    module_id INTEGER NOT NULL,
    cohort TEXT NOT NULL,
    name TEXT NOT NULL,
    instructions TEXT,
    date_created DATE NOT NULL,
    type TEXT NOT NULL,
    time_limit INTEGER, -- IF NULL, NO TIME LIMIT
    active BOOLEAN DEFAULT TRUE,
    pass_percentage DECIMAL(5,2) NOT NULL DEFAULT 50.0,
    FOREIGN KEY (teacher_id) REFERENCES teachers(id) ON DELETE CASCADE,
    FOREIGN KEY (module_id) REFERENCES modules(id) ON DELETE CASCADE
);

-- STUDENT_MODULE Junction Table
CREATE TABLE student_module (
    student_id INTEGER,
    module_id INTEGER,
    cohort TEXT NOT NULL,
    PRIMARY KEY (student_id, module_id),
    FOREIGN KEY (student_id) REFERENCES students(id) ON DELETE CASCADE,
    FOREIGN KEY (module_id) REFERENCES modules(id) ON DELETE CASCADE
);

-- TEACHER_MODULE Junction Table
CREATE TABLE teacher_module (
    teacher_id INTEGER,
    module_id INTEGER,
    PRIMARY KEY (teacher_id, module_id),
    FOREIGN KEY (teacher_id) REFERENCES teachers(id) ON DELETE CASCADE,
    FOREIGN KEY (module_id) REFERENCES modules(id) ON DELETE CASCADE
);


-- STUDENT_TEST Junction Table
CREATE TABLE student_test (
    student_id INTEGER,
    test_id INTEGER,
    PRIMARY KEY (student_id, test_id),
    FOREIGN KEY (student_id) REFERENCES students(id) ON DELETE CASCADE,
    FOREIGN KEY (test_id) REFERENCES tests(id) ON DELETE CASCADE
);

-- TEST_ATTEMPT Table to track test results
CREATE TABLE test_attempt (
    id INTEGER PRIMARY KEY AUTOINCREMENT,
    student_id INTEGER NOT NULL,
    test_id INTEGER NOT NULL,
    score DECIMAL(5,2) NOT NULL, -- Could probably automate this based off submission_answers
    time_taken INTEGER NOT NULL, -- This records how long a student took on a test
    attempt_date TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
    FOREIGN KEY (student_id) REFERENCES students(id) ON DELETE CASCADE,
    FOREIGN KEY (test_id) REFERENCES tests(id) ON DELETE CASCADE
);

-- QUESTION ATTEMPT Table to track question results
CREATE TABLE question_attempt (
    id INTEGER PRIMARY KEY AUTOINCREMENT,
    test_attempt_id INTEGER NOT NULL,
    mcq_id INTEGER, -- if mcq_id is NULL, then fib MUST have id and vice versa
    fib_id INTEGER,
    student_answer TEXT NOT NULL,
    FOREIGN KEY (test_attempt_id) REFERENCES test_attempt(id),
    FOREIGN KEY (mcq_id) REFERENCES MCQ(id),
    FOREIGN KEY (fib_id) REFERENCES FIB(id)

);

CREATE TABLE test_questions (
    id INTEGER PRIMARY KEY AUTOINCREMENT,
    test_id INTEGER NOT NULL,
    question_id INTEGER NOT NULL,
    question_type TEXT NOT NULL, -- 'MCQ' or 'FIB'
    FOREIGN KEY (test_id) REFERENCES tests(id) ON DELETE CASCADE
);

-- TEST_MCQ Junction Table
CREATE TABLE test_mcq (
    test_id INTEGER,
    mcq_id INTEGER,
    PRIMARY KEY (test_id, mcq_id),
    FOREIGN KEY (test_id) REFERENCES tests(id) ON DELETE CASCADE,
    FOREIGN KEY (mcq_id) REFERENCES MCQ(id) ON DELETE CASCADE
);

-- TEST_FIB Junction Table
CREATE TABLE test_fib (
    test_id INTEGER,
    fib_id INTEGER,
    PRIMARY KEY (test_id, fib_id),
    FOREIGN KEY (test_id) REFERENCES tests(id) ON DELETE CASCADE,
    FOREIGN KEY (fib_id) REFERENCES FIB(id) ON DELETE CASCADE
);



