from flask import Flask, session
from app.config import Config 
import os
from app.models import  db
from app.routes import routes_bp
from flask_sqlalchemy import SQLAlchemy
from flask_login import LoginManager
from flask_migrate import Migrate

# db = SQLAlchemy()
login_manager = LoginManager()


def create_app():
    app = Flask(__name__, template_folder=os.path.abspath('app/templates'))
    app.config.from_object(Config)

    # Set up database URI with absolute path
    basedir = os.path.abspath(os.path.dirname(__file__)) 
    if not os.path.exists(basedir):
        os.makedirs(basedir)
    app.config['SQLALCHEMY_DATABASE_URI'] = 'sqlite:///' + os.path.join(basedir, 'your_database_name.db')
    
    # init SQLAlchemy
    db.init_app(app)
    login_manager.init_app(app)

    migrate = Migrate(app, db)
    
    # register Blueprint
    app.register_blueprint(routes_bp) 

    # with app.app_context():
    #     db.create_all()
   
    login_manager.login_view = 'routes.login'
   
    return app

# login_manager init function
@login_manager.user_loader
def load_user(id):
    from app.models import students, teachers
    user = teachers.query.get(int(id))
    if user and session.get("role") == "teacher":
        return user
    return students.query.get(int(id))
    # if not student then teacher
