import os
import sqlite3

# Get the path to the SQL files
base_dir = os.path.dirname(os.path.abspath(__file__))  # Get the current directory
schema_file = os.path.join(base_dir, 'SQLs', 'schema.sql')  # Path to schema.sql inside the SQLs folder
init_file = os.path.join(base_dir, 'SQLs', 'init.sql')

def init_db():
    # Ensure the database is created and connected
    conn = sqlite3.connect('your_database_name.db')  # Use the appropriate database name
    cursor = conn.cursor()

    # Open and execute the schema SQL file
    try:
        # Load schema.sql
        print(f"Reading schema file: {schema_file}")
        with open(schema_file, 'r') as f:
            schema_sql = f.read()
            print("Schema file read successfully.")
            cursor.executescript(schema_sql)  # Execute the schema SQL script
        print("Database schema created successfully.")

        # Now load and execute init.sql to populate the database
        print(f"Reading init file: {init_file}")
        with open(init_file, 'r') as f:
            init_sql = f.read()
            print("Init file read successfully.")
            cursor.executescript(init_sql)  # Execute the data population SQL script
        print("Database populated successfully.")

        # Commit the changes and close the connection
        conn.commit()
        print("Database initialization complete.")

    except FileNotFoundError:
        print(f"Error: {schema_file} or {init_file} not found.")
    except Exception as e:
        print(f"An error occurred: {e}")
    finally:
        conn.close()

if __name__ == "__main__":
    init_db()
