from flask import Blueprint, render_template, jsonify, request, redirect, url_for, flash, session, abort
from flask_login import login_user, login_required, logout_user, current_user
from app.models import  db , TestResult, QuestionAnswer, Feedback, students ,teachers ,FIB, MCQ, TestORM, ModuleORM, TestQuestion
from werkzeug.security import check_password_hash, generate_password_hash
from sqlalchemy import text, bindparam
from sqlalchemy.orm import joinedload
from app.models import *
from datetime import date, datetime

import os
import re


routes_bp = Blueprint("routes", __name__)
print(f"Routes for this blueprint: {routes_bp.name}")


@routes_bp.route("/")
@routes_bp.route("/index")
def root():
    return redirect(url_for("routes.login"))


@routes_bp.route("/student")
@login_required
def student():
    return render_template("student.html")

from werkzeug.security import generate_password_hash, check_password_hash

@routes_bp.route('/login', methods=['GET', 'POST'])
def login():
    if session.get("user_id"):
        return render_template("index.html" if session.get("role") == "teacher" else "student.html")

    if request.method == 'POST':
        username = request.form.get('username')
        password = request.form.get('password')

        role = "teacher" if username.startswith('0') else "student"
        get_user = teachers.get_userinfo_by_username if role == "teacher" else students.get_userinfo_by_username

        user = get_user(username, password, db=db)  # This might return None if password doesn't match

        if not user:
            # Try fetching by username only, to check if password is plaintext and upgrade it
            sql = "SELECT * FROM teachers WHERE username = :u" if role == "teacher" else "SELECT * FROM students WHERE username = :u"
            result = db.session.execute(text(sql), {"u": username}).fetchone()

            if result:
                # Plaintext password upgrade check
                if result.password == password:
                    hashed = generate_password_hash(password)
                    update_sql = "UPDATE teachers SET password = :p WHERE id = :i" if role == "teacher" else "UPDATE students SET password = :p WHERE id = :i"
                    db.session.execute(text(update_sql), {"p": hashed, "i": result.id})
                    db.session.commit()

                    # Try to fetch again after update
                    user = get_user(username, password, db=db)

        if user:
            login_user(user)
            session["user_id"] = user.id
            session["role"] = role
            return render_template("index.html" if role == "teacher" else "student.html")
        else:
            flash("Login failed. Check your username or password again.", "warning")
            return render_template("login.html")

    return render_template("login.html")




@routes_bp.route('/addMCQ', methods = ['GET', 'POST'])
def addMCQ():
    if not session.get("user_id"):
        flash("Please login first.", "warning")
        return redirect(url_for('routes.login'))

    teacher_id = session["user_id"]
    role = session["role"]

    if not role:
        flash("Please login first.", "warning")
        return redirect(url_for('routes.login'))

    if role != "teacher":
        flash("Access Denied.", "danger")
        return redirect(url_for('routes.login'))



    if request.method == 'POST':
    # Extract form data
        difficulty = request.form.get('Difficulty')
        learning_obj = request.form.get('learning_obj')
        module = request.form.get('module')
        question_text = request.form.get('question_text')
        option_A = request.form.get('option_A')
        option_B = request.form.get('option_B')
        option_C = request.form.get('option_C')
        option_D = request.form.get('option_D')
        correct_answer = request.form.get('correct_answer')
        feedback = request.form.get('feedback')

        
        sql = """
                INSERT INTO MCQ 
                (teacher_id, module_id, question, option_A, option_B, option_C, option_D, answer, difficulty, learning_obj, feedback, active) 
                VALUES 
                (:teacher_id, :module_id, :question, :option_A, :option_B, :option_C, :option_D, :answer, :difficulty, :learning_obj, :feedback, :active)
        """
        sql_q = text(sql)


        #Add skill
        db.session.execute(
            sql_q,
            {
            "teacher_id": teacher_id,
            "module_id": module,
            "question": question_text,
            "option_A": option_A,
            "option_B": option_B,
            "option_C": option_C,
            "option_D": option_D,
            "answer": correct_answer,
            "difficulty": difficulty,
            "learning_obj": learning_obj,
            "feedback": feedback,
            "active": 1
                }
        )

        db.session.commit()
        flash("Question added sucessfully.", "success")
        return redirect(url_for('routes.addMCQ'))

    base_sql = """
            SELECT * 
            FROM teachers JOIN teacher_module ON teachers.id = teacher_module.teacher_id 
            JOIN modules ON modules.id = teacher_module.module_id 
            WHERE teachers.id = :teacher_id 
    """
    sql = text(base_sql)
    modules = db.engine.connect().execute(sql, {'teacher_id': teacher_id}).fetchall()
    return render_template("addMCQ.html", modules=modules)


@routes_bp.route('/reviewStudents', methods=['GET', 'POST'])
def reviewStudents():
    if not session.get("user_id") or not session.get("role") or session["role"] != "teacher":
        flash("Access Denied. Please login as a teacher.", "warning")
        return redirect(url_for('routes.login'))

    student_id = request.args.get('student_id')
    test_id = request.args.get('test_id')
    name = request.args.get('name')

    test_attempts = TestDAO.getAttemptsByTest(test_id, db)

    # Filter and sort by date (oldest to newest)
    filtered_attempts = [a for a in test_attempts if str(a["student_id"]) == str(student_id)]
    filtered_attempts.sort(key=lambda x: x["attempt_date"])

    # Add attempt_id (1st attempt = 1, etc.)
    for i, attempt in enumerate(filtered_attempts, start=1):
        attempt["attempt_id"] = i

    return render_template(
        'reviewStudents.html',
        student_attempts=filtered_attempts,
        student_name=name
    )

@routes_bp.route('/reviewResults', methods = ['GET', 'POST'])
def reviewResults():
    if not session.get("user_id"):
        flash("Please login first.", "warning")
        return redirect(url_for('routes.login'))
    
    role = session["role"]

    if not role:
        flash("Please login first.", "warning")
        return redirect(url_for('routes.login'))

    if role != "teacher":
        flash("Access Denied.", "danger")
        return redirect(url_for('routes.login'))

    # SELECT MODULE - displays modules for that teacher
    modules = teachers.getModules(session.get("user_id"), db)

    # SELECT COHORT - display a list of cohorts
    # SELECTS TESTS - display a list of tests given to that cohort
    # SELECTS APPROPRIATE STUDENTS / QUESTIONS - display students / questions for that test ( independant of cohort )
    # UPDATES TABLE ONCE ALL THOSE ARE DONE
    return render_template('reviewResults.html', modules = modules)

@routes_bp.route('/getCohorts', methods = ['POST'])
def getCohorts():
    data = request.get_json()
    module_id = data.get("module_id")

    if not module_id:
        return jsonify({"error": "Module ID is required"}), 400

    # Fetch cohorts using the ModuleDAO method
    cohorts = ModuleDAO.getCohortsByModule(module_id, db)

    # Return cohorts as JSON response
    return jsonify(cohorts)

@routes_bp.route('/getTests', methods=['POST'])
def getTests():
    data = request.get_json()
    module_id = data.get("module_id")
    cohort = data.get("cohort")

    if not module_id or not cohort:
        return jsonify({"error": "Module ID and Cohort are required"}), 400

    tests = TestDAO.getTestsByModuleAndCohort(module_id, cohort, db)
    return jsonify(tests)

#get students and questions when a test is selected
@routes_bp.route('/getTestDetails', methods=['POST'])
def getTestDetails():
    data = request.get_json()
    test_id = data.get("test_id")
    attempt_type = data.get("attempt_type", "Recent")  # Default to Best if not provided

    if not test_id:
        return jsonify({"error": "Test ID is required"}), 400

    # Get all test questions for this test_id
    stmt = text("""
        SELECT question_id, question_type
        FROM test_questions
        WHERE test_id = :test_id
    """)
    result = db.session.execute(stmt, {"test_id": test_id}).fetchall()

    # Split question IDs by type
    mcq_ids = [row[0] for row in result if row[1] == 'MCQ']
    fib_ids = [row[0] for row in result if row[1] == 'FIB']

    mcq_questions = []
    fib_questions = []

    if mcq_ids:
        mcq_stmt = text("""
            SELECT id, question, option_A, option_B, option_C, option_D, answer, difficulty, learning_obj
            FROM MCQ
            WHERE id IN :mcq_ids
        """).bindparams(bindparam("mcq_ids", expanding=True))
        
        mcq_results = db.session.execute(mcq_stmt, {"mcq_ids": mcq_ids}).fetchall()
        mcq_questions = [
            {
                "id": row[0],
                "question": row[1],
                "options": [row[2], row[3], row[4], row[5]],
                "answer": row[6],
                "difficulty": ("Difficulty." + str(row[7])) if row[7] else None,
                "learning_obj": ("LearningObjective." + str(row[8])) if row[8] else None,
            }
            for row in mcq_results
        ]

    if fib_ids:
        fib_stmt = text("""
            SELECT id, question, answer, difficulty, learning_obj
            FROM FIB
            WHERE id IN :fib_ids
        """).bindparams(bindparam("fib_ids", expanding=True))

        fib_results = db.session.execute(fib_stmt, {"fib_ids": fib_ids}).fetchall()

        fib_questions = [
            {
                "id": row[0],
                "question": row[1],
                "answer": row[2],
                "difficulty": ("Difficulty." + str(row[3])) if row[3] else None,
                "learning_obj": ("LearningObjective." + str(row[4])) if row[4] else None,
            }
            for row in fib_results
        ]

    # Get students and all attempts
    students = TestDAO.getStudentsByTest(test_id, db)
    test_attempts = TestDAO.getAttemptsByTest(test_id, db)

    # Determine which attempt to use per student
    valid_types = {"Best", "First", "Recent"}
    attempt_type = attempt_type if attempt_type in valid_types else "Best"

    student_attempts_map = {}

    for attempt in test_attempts:
        student_id = attempt["student_id"]
        current = student_attempts_map.get(student_id)

        if attempt_type == "Best":
            if not current or attempt["score"] > current["score"]:
                student_attempts_map[student_id] = attempt

        elif attempt_type == "First":
            if not current or attempt["attempt_date"] < current["attempt_date"]:
                student_attempts_map[student_id] = attempt

        elif attempt_type == "Recent":
            if not current or attempt["attempt_date"] > current["attempt_date"]:
                student_attempts_map[student_id] = attempt

    filtered_attempts = list(student_attempts_map.values())

    return jsonify({
        'mcq_questions': mcq_questions,
        'fib_questions': fib_questions,
        'students': students,
        'test_attempts': filtered_attempts
    })


@routes_bp.route('/getStudentQuestions', methods=['POST'])
def getStudentQuestions():
    data = request.get_json()
    test_attempt_id = data.get("test_attempt_id")
    test_id = data.get("test_id")

    # Retrieve the question attempts for this test attempt
    question_attempts = TestAttempt.getQuestionAttempts(test_attempt_id, db)

    # Retrieve all test questions with their details in one unified query
    test_questions_stmt = text("""
        SELECT tq.question_id, tq.question_type,
               mcq.question AS mcq_question, mcq.answer AS mcq_answer,
               fib.question AS fib_question, fib.answer AS fib_answer
        FROM test_questions tq
        LEFT JOIN MCQ mcq ON tq.question_type = 'MCQ' AND tq.question_id = mcq.id
        LEFT JOIN FIB fib ON tq.question_type = 'FIB' AND tq.question_id = fib.id
        WHERE tq.test_id = :test_id
    """)

    question_rows = db.session.execute(test_questions_stmt, {"test_id": test_id}).fetchall()

    # Map questions by (type, id)
    question_lookup = {}
    for row in question_rows:
        q_id = row[0]
        q_type = row[1]
        question = row[2] if q_type == "MCQ" else row[4]
        answer = row[3] if q_type == "MCQ" else row[5]

        question_lookup[(q_type, q_id)] = {
            "question": question,
            "answer": answer,
            "id": q_id,
            "type": q_type
        }

    # Build response
    serialized_attempts = []
    for attempt in question_attempts:
        if attempt.mcq_id:
            key = ("MCQ", attempt.mcq_id)
        elif attempt.fib_id:
            key = ("FIB", attempt.fib_id)
        else:
            continue  # Skip if no valid question ID

        question_data = question_lookup.get(key, {})

        serialized_attempts.append({
            "id": question_data.get("id"),
            "test_attempt_id": attempt.test_attempt_id,
            "correct_answer": question_data.get("answer"),
            "student_answer": attempt.student_answer,
            "question_type": question_data.get("type", "").lower(),
            "question": question_data.get("question")
        })

    return jsonify({"question_attempts": serialized_attempts})



@routes_bp.route('/createTests', methods=['GET', 'POST'])
def createTests():
    tests = TestORM.query.all()
    return render_template('createTest.html', tests=tests)

@routes_bp.route('/buildTests', methods=['GET', 'POST'])
def buildTests():
    fib_questions = db.session.execute(text("""
        SELECT fib.id, fib.question, fib.learning_obj, modules.code AS module_code
        FROM FIB
        JOIN modules ON fib.module_id = modules.id
    """)).fetchall()

    # Updated SQL for MCQ with module code
    mcq_questions = db.session.execute(text("""
        SELECT mcq.id, mcq.question, mcq.learning_obj, modules.code AS module_code
        FROM MCQ
        JOIN modules ON mcq.module_id = modules.id
    """)).fetchall()
    tests = TestORM.query.all()

    if request.method == "POST":
        
        test_type = request.form.get("q-type")
        time_limit = request.form.get("time_limit")
        module_code = request.form.get("learning")
        time_limit = request.form.get("time_limit")
        pass_percentage = request.form.get("pass_percentage")  
        test_title = request.form.get("testform")
        instructions = request.form.get("skill_content")

        if not (test_title and instructions and test_type and time_limit and module_code and pass_percentage):
            flash("Please fill out all required fields.", "danger")
            return redirect(url_for("routes_bp.buildTests"))
        
        module = ModuleORM.query.filter_by(code=module_code).first()

        if not module:
            flash(f"Module with code {module_code} not found.", "danger")
            return redirect(url_for("routes_bp.buildTests"))

        # Create the test
        new_test = TestORM(
            teacher_id=current_user.id,
            name=test_title,
            type=test_type,
            module_id=module.id,  
            date_created=date.today(),
            pass_percentage=float(pass_percentage),  
            active=True,
            time_limit=int(time_limit)
        )

        db.session.add(new_test)
        db.session.commit()

        selected_questions = request.form.getlist("selected_questions")  

        for question_id in selected_questions:
            
            question_type = request.form.get(f"question_type_{question_id}")  
            if question_type == "MCQ":
                question = MCQ.query.get(question_id)
            elif question_type == "FIB":
                question = FIB.query.get(question_id)

           
            test_question = TestQuestion(
                test_id=new_test.id,  
                question_id=question.id,  
                question_type=question_type
            )
            db.session.add(test_question)
        
        db.session.commit()  

        flash("Test created successfully!", "success")
        return redirect(url_for("routes_bp.createTests"))

    return render_template('buildTest.html', fib_questions=fib_questions, mcq_questions=mcq_questions, tests=tests)



@routes_bp.route('/get-modules', methods=['GET'])
def get_modules():
    modules = ModuleDAO.allModules(db)  
    module_list = [{"id": module.id, "code": module.code, "title": module.title} for module in modules]
    return jsonify(module_list)  
  

@routes_bp.route("/save-test", methods=["POST"])
def save_test():
    data = request.get_json()
    if not data:
        return jsonify({"error": "No JSON data received"}), 400

    teacher_id = session.get("user_id")
    role = session.get("role")
    if not teacher_id or role != "teacher":
        return jsonify({"error": "Unauthorized. Only teachers can create tests."}), 403

    module = ModuleORM.query.get(int(data.get("module_code")))

    if not module:
        return jsonify({"error": f"Module '{data.get('module_code')}' not found."}), 400

    name = data.get("name", "").strip()
    cohort = data.get("cohort", "").strip()
    instructions = data.get("instructions", "")
    test_type = data.get("test_type", "")
    time_limit = data.get("time_limit", "")
    pass_percentage = float(data.get("pass_percentage", 0))
    questions = data.get("questions", [])
    questions_to_delete = data.get("questions_to_delete", [])

    if not name or not cohort:
        return jsonify({"error": "Test name and cohort are required."}), 400

    test_id = data.get("test_id")

    try:
        if test_id:
            # -----------------------
            # UPDATE EXISTING TEST
            # -----------------------
            test = TestORM.query.get(test_id)
            if not test:
                return jsonify({"error": "Test not found."}), 404

            test.name = name
            test.cohort = cohort
            test.instructions = instructions
            test.type = test_type
            test.time_limit = time_limit
            test.pass_percentage = pass_percentage
            test.module_id = module.id

            # 1. Delete unwanted question links first
            questions_to_delete = data.get("questions_to_delete", [])
            questions_to_delete = [
                (int(q.split("-")[0]), q.split("-")[1]) for q in questions_to_delete
            ]  # Ensure type match
            print(f"Trying to delete: {questions_to_delete} from test {test.id}")

            if questions_to_delete:
                for qid, qtype in questions_to_delete:
                    TestQuestion.query.filter_by(
                        test_id=test.id,
                        question_id=qid,
                        question_type=qtype
                    ).delete(synchronize_session='fetch')


            # 2. Fetch existing question IDs *after deletion*
            existing_links = {
                (tq.test_id, tq.question_id, tq.question_type)
                for tq in TestQuestion.query.filter_by(test_id=test.id).all()
            }

            print("Existing links:", existing_links)
            print("Questions sent from frontend:", questions)

            for q in questions:
                if not isinstance(q, dict):
                    print(f"⚠️ Skipping malformed question entry: {q}")
                    continue

                qid = int(q.get("question_id"))
                qtype = q.get("type")

                print(f"Looping q: {q}")
                print(f"Comparing: (test.id={test.id}, question_id={qid}, type={qtype})")

                if (test.id, qid, qtype) in existing_links:
                    print(f"🟡 Already exists — skipping ({test.id}, {qid}, {qtype})")
                else:
                    print(f"🟢 Adding NEW link: ({test.id}, {qid}, {qtype})")
                    db.session.add(TestQuestion(test_id=test.id, question_id=qid, question_type=qtype))



            db.session.commit()
            return jsonify({"message": "Test updated successfully", "test_id": test.id}), 201

        else:
            # -----------------------
            # CREATE NEW TEST
            # -----------------------
            new_test = TestORM(
                teacher_id=teacher_id,
                module_id=module.id,
                cohort=cohort,
                name=name,
                instructions=instructions,
                date_created=datetime.utcnow(),
                type=test_type,
                time_limit=time_limit,
                pass_percentage=pass_percentage
            )
            db.session.add(new_test)
            db.session.commit()
            print("Creating test — questions from frontend:", questions)
            for q in questions:
                qid = q.get("question_id")
                qtype = q.get("type")
                if qid:
                    tq = TestQuestion(test_id=new_test.id, question_id=qid, question_type=qtype)
                    db.session.add(tq)

            db.session.commit()
            return jsonify({"message": "Test created successfully", "test_id": new_test.id}), 201

    except Exception as e:
        db.session.rollback()
        print("Error saving test:", e)
        return jsonify({"error": "An error occurred while saving the test."}), 500




@routes_bp.route('/delete_test/<int:test_id>', methods=['GET', 'POST'])
def delete_test(test_id):
    # Ensure user is logged in and is a teacher
    teacher_id = session.get("user_id")
    role = session.get("role")

    if not teacher_id or role != "teacher":
        flash("Unauthorized. Only teachers can delete tests.", "danger")
        return redirect(url_for("routes.createTests"))

    # Get the test to be deleted
    test_to_delete = TestORM.query.get(test_id)

    if not test_to_delete:
        flash("Test not found.", "danger")
        return redirect(url_for("routes.createTests"))

    # Ensure the current user is the owner of the test (i.e., the teacher who created it)
    if test_to_delete.teacher_id != teacher_id:
        flash("You can only delete tests that you have created.", "danger")
        return redirect(url_for("routes.createTests"))

    # Delete the test and commit the changes to the database
    db.session.delete(test_to_delete)
    db.session.commit()

    flash("Test deleted successfully!", "success")
    return redirect(url_for("routes.createTests"))

@routes_bp.route('/editTest/<int:test_id>', methods=['GET', 'POST'])
def editTest(test_id):
    # Ensure user is logged in
    if not session.get("user_id"):
        flash("Please login first.", "warning")
        return redirect(url_for('routes.login'))
    
    role = session["role"]
    # Ensure the user is a teacher
    if role != "teacher":
        flash("Access Denied.", "danger")
        return redirect(url_for('routes.login'))
    
    teacher_id = session["user_id"]
    # Validate Test Ownership: Make sure the logged-in teacher owns the test
    sql_validate = "SELECT * FROM tests WHERE id = :test_id AND teacher_id = :teacher_id"
    test_data = db.session.execute(text(sql_validate), {'test_id': test_id, 'teacher_id': teacher_id}).fetchone()
    if not test_data:
        abort(404, description="Test not found.")
    
    sql_test = """
    SELECT tests.*, modules.id AS module_id, modules.code AS module_code
    FROM tests
    JOIN modules ON tests.module_id = modules.id
    WHERE tests.id = :test_id
    """
    current_test = db.session.execute(text(sql_test), {'test_id': test_id}).fetchall()
    
    # Fetch current questions in the test
    sql_questions = """
    SELECT tq.id, tq.test_id, tq.question_id, tq.question_type, 
    CASE 
        WHEN tq.question_type = 'MCQ' THEN mcq.question
        WHEN tq.question_type = 'FIB' THEN fib.question
    END AS question_text,
    CASE 
        WHEN tq.question_type = 'MCQ' THEN m1.code
        WHEN tq.question_type = 'FIB' THEN m2.code
    END AS module_code
    FROM test_questions tq
    LEFT JOIN MCQ mcq ON tq.question_id = mcq.id AND tq.question_type = 'MCQ'
    LEFT JOIN modules m1 ON mcq.module_id = m1.id
    LEFT JOIN FIB fib ON tq.question_id = fib.id AND tq.question_type = 'FIB'
    LEFT JOIN modules m2 ON fib.module_id = m2.id
    WHERE tq.test_id = :test_id
    """
    current_questions = db.session.execute(text(sql_questions), {'test_id': test_id}).fetchall()
    
    # Fetch all available MCQ and FIB questions
    sql_all_questions = """
    SELECT mcq.id AS id, mcq.question, 'MCQ' AS type, modules.code AS module_code
    FROM MCQ
    JOIN modules ON mcq.module_id = modules.id
    UNION ALL
    SELECT fib.id AS id, fib.question, 'FIB' AS type, modules.code AS module_code
    FROM FIB
    JOIN modules ON fib.module_id = modules.id
    """
    all_questions = db.session.execute(text(sql_all_questions)).fetchall()

    
    if request.method == 'POST':
        # Get updated test details from the form
        test_name = request.form.get('name')
        test_type = request.form.get('q-type')
        time_limit = request.form.get('time_limit')
        module_code = request.form.get('learning')
        pass_percentage = request.form.get('pass_percentage')
        cohort = request.form.get('cohort')
        instructions = request.form.get('skill_content')
        
        # Get selected question IDs
        selected_questions = request.form.getlist('questions')
        
        # Update the test details in the database
        sql_update_test = """
        UPDATE tests 
        SET name = :name, test_type = :test_type, time_limit = :time_limit, 
        module_id = :module_id, pass_percentage = :pass_percentage, 
        cohort = :cohort, instructions = :instructions
        WHERE id = :test_id
        """
        db.session.execute(text(sql_update_test), {
            'name': test_name,
            'test_type': test_type,
            'time_limit': time_limit,
            'module_id': module_code,
            'pass_percentage': pass_percentage,
            'cohort': cohort,
            'instructions': instructions,
            'test_id': test_id
        })
        db.session.commit()
        
        # Update the questions associated with the test
        # First, delete existing questions
        sql_delete_questions = "DELETE FROM test_questions WHERE test_id = :test_id"
        db.session.execute(text(sql_delete_questions), {'test_id': test_id})
        
        # Then, add the selected questions
        for question_id in selected_questions:
            question_type = request.form.get(f'question_type_{question_id}')
            sql_insert_question = """
            INSERT INTO test_questions (test_id, question_id, question_type)
            VALUES (:test_id, :question_id, :question_type)
            """
            db.session.execute(text(sql_insert_question), {
                'test_id': test_id,
                'question_id': question_id,
                'question_type': question_type
            })
        db.session.commit()
        
        flash("Test updated successfully!", "success")
        return redirect(url_for('routes.editTest', test_id=test_id))
    
    return render_template('editTest.html', current_questions=current_questions, all_questions=all_questions, current_test=current_test)

@routes_bp.route('/questionBank')
def questionBank():
    if not session.get("user_id"):
        flash("Please login first.", "warning")
        return redirect(url_for('routes.login'))

    role = session["role"]

    if role != "teacher":
        flash("Access Denied.", "danger")
        return redirect(url_for('routes.login'))

    user_id = session["user_id"]
    difficulty = request.args.get('sort-by-dif')
    learning_obj = request.args.get('sort-by-obj')

    base_sql = "SELECT question, code, difficulty, MCQ.id as mcq_id, modules.id as module_id, learning_obj FROM MCQ JOIN modules ON MCQ.module_id = modules.id WHERE teacher_id = :user_id AND active = 1"
    params = {'user_id': user_id}

    if difficulty:
        base_sql += " AND difficulty = :difficulty"
        params['difficulty'] = difficulty

    if learning_obj:
        base_sql += " AND learning_obj = :learning_obj"
        params['learning_obj'] = learning_obj

    sql = text(base_sql)
    user_questions = db.engine.connect().execute(sql, params).fetchall()

    return render_template('questionBank.html', user_questions=user_questions, difficulty=difficulty, learning_obj = learning_obj)

#DELETE Question from QB
@routes_bp.route('/questionBank/delete/<int:id>', methods = ['POST'])
def deleteMCQ(id):
    teacher_id = session["user_id"]
    if request.method == 'POST':    
        teacher_id = session["user_id"]
        #Deactivate question
        sql = """
                UPDATE MCQ SET active = 0
                WHERE teacher_id = :teacher_id AND id = :id
            """
        sql_q = text(sql)

        db.session.execute(
            sql_q,
            {
            "teacher_id": teacher_id,
            "id": id
                }
        )

        db.session.commit()
        flash("Question deleted sucessfully.", "success")
        return redirect(url_for('routes.questionBank'))

#Edit Question from QB
@routes_bp.route('/questionBank/editMCQ/<int:id>', methods = ['GET', 'POST'])
def editMCQ(id):
    if not session.get("user_id"):
        flash("Please login first.", "warning")
        return redirect(url_for('routes.login'))

    role = session["role"]

    if role != "teacher":
        flash("Access Denied.", "danger")
        return redirect(url_for('routes.login'))

    teacher_id = session["user_id"]

    #Validate active and teacher_id
    sql_validate = "SELECT teacher_id, active FROM MCQ WHERE id = :id"
    sql_validate_q = text(sql_validate)
    result = db.engine.connect().execute(sql_validate_q, {'id':id}).fetchone()
    if not result:
        abort(404, description="Question not found.")
        return redirect(url_for('routes.questionBank'))

    if result.active == 0:
        flash("Question not found.", "danger")
        return redirect(url_for('routes.questionBank'))

    if result.teacher_id != teacher_id:
        flash("Invalid permission.", "danger")
        return redirect(url_for('routes.questionBank'))


    if request.method == 'POST':
        difficulty = request.form.get('Difficulty')
        learning_obj = request.form.get('learning_obj')
        module = request.form.get('module')
        question_text = request.form.get('question_text')
        option_A = request.form.get('option_A')
        option_B = request.form.get('option_B')
        option_C = request.form.get('option_C')
        option_D = request.form.get('option_D')
        correct_answer = request.form.get('correct_answer')
        feedback = request.form.get('feedback')

        sql = """
            UPDATE MCQ
            SET module_id = :module_id,
                question = :question,
                option_A = :option_A,
                option_B = :option_B,
                option_C = :option_C,
                option_D = :option_D,
                answer = :answer,
                difficulty = :difficulty,
                learning_obj = :learning_obj,
                feedback = :feedback
            WHERE id = :id
        """
        sql_q = text(sql)


        #Update question
        db.session.execute(
            sql_q,
            {
            "module_id": module,
            "question": question_text,
            "option_A": option_A,
            "option_B": option_B,
            "option_C": option_C,
            "option_D": option_D,
            "answer": correct_answer,
            "difficulty": difficulty,
            "learning_obj": learning_obj,
            "feedback": feedback,
            "id": id
                }
        )

        db.session.commit()
        flash("Question updated sucessfully.", "Sucess")
        return redirect(url_for('routes.questionBank'))

    
    #Query for selected question
    sql = """
        SELECT question, code, difficulty, MCQ.id as mcq_id, modules.id as module_id, option_A, option_B, option_C, option_D, answer , learning_obj, feedback 
        FROM MCQ JOIN modules ON MCQ.module_id = modules.id 
        WHERE teacher_id = :teacher_id AND MCQ.id = :id
        """
    sql_q = text(sql)
    question = db.engine.connect().execute(sql_q, {'teacher_id':teacher_id, 'id':id}).fetchone()

    sql2 = "SELECT code, m.id FROM teacher_module tm JOIN modules m ON tm.module_id = m.id WHERE tm.teacher_id = :teacher_id"
    sql2_q = text(sql2)
    modules = db.engine.connect().execute(sql2_q, {'teacher_id':teacher_id}).fetchall()

    db.session.commit()
    return render_template('editMCQ.html', question=question, modules=modules)

@routes_bp.route('/fill_the_blanks', endpoint='fill_the_blanks')
def fill_the_blanks():
    difficulty = request.args.get('sort-by-dif', '')
    learning_obj = request.args.get('sort-by-obj', '')

    query = FIB.query.filter_by(active=True)

    if difficulty:
        query = query.filter(FIB.difficulty == difficulty)
    if learning_obj:
        query = query.filter(FIB.learning_obj == learning_obj)

    fib_questions = query.all()
    formatted_questions = []
    for q in fib_questions:
        formatted_questions.append({
            "id": q.id,
            "module_id": q.module_id,
            "question": q.question,
            "answer": q.answer,
            "difficulty": q.difficulty.value if q.difficulty else "",
            "learning_obj": q.learning_obj.value if q.learning_obj else ""
        })

    total_questions = len(formatted_questions)
    return render_template(
        "fillTheBlanks.html",
        questions=formatted_questions,
        total_questions=total_questions,
        difficulty=difficulty,
        learning_obj=learning_obj
    )

@routes_bp.route('/addFIB', methods=['GET', 'POST'])
@login_required   
def addFIB():
    teacher_id = session["user_id"]

    if request.method == 'POST': 
        # teacher_id = request.form.get('teacher_id')
        module_id = request.form.get('module')
        # question = request.form.get('new_question')
        question = re.sub(r'<[^>]+>', '', request.form.get('new_question'))
        answer = request.form.get('answer')
        difficulty = request.form.get('difficulty')
        learning_obj = request.form.get('learning_obj')
        feedback = request.form.get('feedback')
        # insert into FIB
        sql = """
            INSERT INTO FIB 
            (teacher_id, module_id, question, answer, difficulty, learning_obj, active, feedback) 
            VALUES 
            (:teacher_id, :module_id, :question, :answer, :difficulty, :learning_obj, :active, :feedback)
        """
        sql_q = text(sql)
 
        db.session.execute(
            sql_q,
            {
                "teacher_id": teacher_id,
                "module_id": module_id,
                "question": question,
                "answer": answer,
                "difficulty": difficulty,
                "learning_obj": learning_obj,
                "feedback": feedback,
                "active": 1
            }
        )

        db.session.commit()
        flash('Successfully added a new fill the blanks question!', 'success')
        return redirect(url_for('routes.fill_the_blanks'))
    base_sql = """
            SELECT * 
            FROM teachers JOIN teacher_module ON teachers.id = teacher_module.teacher_id 
            JOIN modules ON modules.id = teacher_module.module_id 
            WHERE teachers.id = :teacher_id 
    """
    sql = text(base_sql)
    modules = db.engine.connect().execute(sql, {'teacher_id': teacher_id}).fetchall()
    return render_template("addFIB.html", modules=modules)

@routes_bp.route('/editFIB/<int:id>', methods=['GET', 'POST'])
# @login_required
def editFIB(id):
    teacher_id = session["user_id"]
    fib = db.session.query(FIB).filter_by(id=id, teacher_id=teacher_id).first()

    if not fib: 
        flash("Your role is not a teacher, or the question is not created by you. Please select another question!", "warning")
        return redirect(url_for('routes.fill_the_blanks'))

    if request.method == 'POST':
        fib.question = strip_html_tags(request.form.get('new_question'))
        fib.answer = request.form.get('answer')
        fib.difficulty = request.form.get('difficulty')
        fib.learning_obj = request.form.get('learning_obj')
        fib.module_id = request.form.get('module')
        fib.feedback = request.form.get('feedback')
        
        db.session.commit()
        flash("Question updated successfully!", "success")
        return redirect(url_for('routes.fill_the_blanks'))

    # Get module drop-down menu information
    base_sql = """
            SELECT * 
            FROM teachers JOIN teacher_module ON teachers.id = teacher_module.teacher_id 
            JOIN modules ON modules.id = teacher_module.module_id 
            WHERE teachers.id = :teacher_id 
    """
    sql = text(base_sql)
    modules = db.engine.connect().execute(sql, {'teacher_id': teacher_id}).fetchall()
    
    return render_template("addFIB.html", fib=fib, modules=modules, mode="edit")

@routes_bp.route('/deleteFIB/<int:id>', methods=['POST'])
def deleteFIB(id):
    fib = FIB.query.get_or_404(id)
    fib.active = False
    db.session.commit()
    flash("Question deleted successfully!", "success")
    return redirect(url_for('routes.fill_the_blanks'))


# @routes_bp.route('/fillTheBlanks')
# def fillTheBlanks(): 
    # module = request.args.get('module', 'cmt313')  
    # sort_by = request.args.get('sort-by', 'date')   
 
    # questions = Question.query.filter_by(type='cr').all()   
    # total_questions = len(questions)

    # return render_template('fill_the_blanks.html', questions=questions, total_questions=total_questions, module=module, sort_by=sort_by)
    # return render_template('fillTheBlanks.html')

# Feedback review(Summative feedback) by CHRIS LYU
@routes_bp.route('/teacherFeedback')
def teacher_feedback():
    
    students_list = db.session.execute(
        db.select(students).options(joinedload(students.test_results))
    ).unique().scalars().all()

    
    all_tests = db.session.execute(
        db.select(TestResult.id).distinct().order_by(TestResult.id)
    ).scalars().all()

    return render_template('teacherFeedback.html', students=students_list, all_tests=all_tests)


@routes_bp.route('/get_feedback/<int:student_id>/<int:test_id>')
def get_feedback(student_id, test_id):
    test_result = TestResult.query.filter_by(student_id=student_id, id=test_id).first()

    if not test_result:
        return jsonify({"error": "Test result not found"}), 404

    return jsonify({
        "test_id": test_result.id,
        "score": test_result.score,
        "feedback": test_result.feedback.comments if test_result.feedback else "",
        "questions": [{
            "question": qa.question_text,
            "student_answer": qa.student_answer,
            "correct_answer": qa.correct_answer,
            "correct": qa.student_answer == qa.correct_answer
        } for qa in test_result.questions]
    })


@routes_bp.route('/submit_feedback', methods=['POST'])
def submit_feedback():
    data = request.json
    test_result_id = data.get('test_result_id')
    comments = data.get('comments')

    
    feedback = Feedback.query.filter_by(test_result_id=test_result_id).first()
    
    if feedback:
        feedback.comments = comments
    else:
        feedback = Feedback(test_result_id=test_result_id, comments=comments)
        db.session.add(feedback)

    db.session.commit()
    return jsonify({'message': 'Feedback saved successfully'})
# End of Feedback review


# This is student side page
@routes_bp.route('/attemptTest')
def attemptTest():
    student_id = session["user_id"]
    role = session["role"]

    sql = """
        SELECT t.id AS testid, t.module_id, t.cohort, t.name, t.instructions, t.date_created, type, time_limit, active, pass_percentage, m.code, m.title   
        FROM tests t JOIN student_module sm ON t.module_id = sm.module_id AND t.cohort= sm.cohort 
        JOIN modules m ON m.id = sm.module_id
        WHERE active = :active AND student_id = :student_id
    """

    sql = text(sql)
    tests = db.engine.connect().execute(sql, {"active": 1, "student_id": student_id}).fetchall()
    return render_template('attemptTest.html', tests=tests)

# This is studetn attempt formative test
@routes_bp.route('/attemptFormative/<int:id>', methods=['GET', 'POST'])
def attemptFormative(id):
    student_id = session["user_id"]
    test_id = id

    if request.method == 'POST':
        answers = request.form.to_dict(flat=True)

        result = {
            "mcq": [],
            "fib": []
        }

        total_questions = 0
        correct_answers = 0

        for key, user_answer in answers.items():
            if '_' in key:
                qtype, qid = key.split('_')
                qid = int(qid)

                if qtype == 'mcq':
                    row = MCQ.query.get(qid)
                    if row:
                        options_map = {
                            'A': row.option_a,
                            'B': row.option_b,
                            'C': row.option_c,
                            'D': row.option_d
                        }

                        user_letter = user_answer.strip().upper()
                        user_option_value = options_map.get(user_letter, '')
                        is_correct = user_letter == row.answer.strip().upper()

                        result["mcq"].append({
                            "question_id": qid,
                            "your_answer": user_option_value,
                            "your_answer_letter": user_letter,
                            "answer": options_map.get(row.answer.strip().upper(), ''),
                            "is_correct": is_correct,
                            "feedback": row.feedback if row.feedback else ''
                        })

                        total_questions += 1
                        if is_correct:
                            correct_answers += 1

                elif qtype == 'fib':
                    row = FIB.query.get(qid)
                    if row:
                        is_correct = user_answer.strip().lower() == row.answer.strip().lower()

                        result["fib"].append({
                            "question_id": qid,
                            "your_answer": user_answer,
                            "answer": row.answer,
                            "is_correct": is_correct,
                            "feedback": row.feedback if row.feedback else ''
                        })

                        total_questions += 1
                        if is_correct:
                            correct_answers += 1

        # Calculate score
        score = round((correct_answers / total_questions) * 100, 2) if total_questions > 0 else 0.0

        # Create test_attempt record
        new_attempt = TestAttempt(
            student_id=student_id,
            test_id=test_id,
            score=score,
            time_taken=0
        )
        new_attempt.attempt_date = datetime.now()
        db.session.add(new_attempt)
        db.session.commit()  # Commit to generate ID for test_attempt

        for key, user_answer in answers.items():
            if '_' in key:
                qtype, qid = key.split('_')
                qid = int(qid)

                if qtype == 'mcq':
                    row = MCQ.query.get(qid)
                    if row:
                        options_map = {
                            'A': row.option_a,
                            'B': row.option_b,
                            'C': row.option_c,
                            'D': row.option_d
                        }

                        user_letter = user_answer.strip().upper()
                        user_option_value = options_map.get(user_letter, '')
                        is_correct = user_letter == row.answer.strip().upper()

                        attempt = QuestionAttempt(
                        test_attempt_id=new_attempt.id,
                        mcq_id=qid,
                        fib_id=None,
                        student_answer=user_letter
                        )
                        db.session.add(attempt)
                        db.session.commit()

                elif qtype == 'fib':
                    row = FIB.query.get(qid)
                    if row:
                        is_correct = user_answer.strip().lower() == row.answer.strip().lower()

                        attempt = QuestionAttempt(
                        test_attempt_id=new_attempt.id,
                        mcq_id=None,
                        fib_id=qid,
                        student_answer=user_answer
                    )
                    db.session.add(attempt)
                    db.session.commit()




        # Store feedback in session and redirect
        session["formative_result"] = result
        return redirect(url_for("routes.formative_answer"))


    # GET method: Render the test page
    sql_test = text("SELECT * FROM tests WHERE id = :id")
    test = db.engine.connect().execute(sql_test, {"id": id}).fetchone()

    sql_mcq = text("""
        SELECT * FROM test_questions tq JOIN MCQ on tq.question_id = MCQ.id
        WHERE test_id = :test_id AND tq.question_type = "MCQ"
    """)
    mcq_questions = db.engine.connect().execute(sql_mcq, {"test_id": id}).fetchall()

    sql_fib = text("""
        SELECT * FROM test_questions tq JOIN FIB on tq.question_id = FIB.id
        WHERE test_id = :test_id AND tq.question_type = "FIB"
    """)
    fib_questions = db.engine.connect().execute(sql_fib, {"test_id": id}).fetchall()

    return render_template('attemptFormative.html', test=test, mcq_questions=mcq_questions, fib_questions=fib_questions)

# formativeAnswer page by Kris
@routes_bp.route('/formativeAnswer')
def formative_answer():
    result = session.get("formative_result", {"mcq": [], "fib": []})
    return render_template("formativeAnswer.html", result=result)



@routes_bp.route('/reviewTests')
def reviewTests():
    return render_template('reviewTests.html')

@routes_bp.route('/submitFeedback')
def submitFeedback():
    return render_template('submitFeedback.html')


@routes_bp.route("/logout")
@login_required
def logout():
    logout_user()
    session.clear()
    return redirect(url_for("routes.login"))


def strip_html_tags(text):
    clean = re.compile('<.*?>')
    return re.sub(clean, '', text).strip()
