document.addEventListener("DOMContentLoaded", function () {
    const addQuestionButton = document.querySelector(".modal-footer .btn-primary");
    const addTestButton = document.querySelector("#addTestButton");
    const questionListTable = document.querySelector("#questionListBody");
    const checkboxes = document.querySelectorAll(".question-checkbox");
    const form = document.querySelector("form");
    const moduleDropdown = document.querySelector("select[name='learning']");
  
    let selectedQuestions = new Map(); // Key = question_id + "-" + type
    let questionCounter = 1;
    const deletedQuestionIds = new Set();
  
    // Load available modules
   
  
    // Load existing questions into the table
    function loadExistingQuestions() {
        const existingRows = document.querySelectorAll("#questionListBody tr");
      
        existingRows.forEach((row) => {
          const questionId = row.getAttribute("data-id");         // From MCQ or FIB
          const questionType = row.getAttribute("data-type");     // 'MCQ' or 'FIB'
          const questionText = row.cells[1].innerText;
          const module = row.cells[3].innerText;
      
          const questionKey = questionId + "-" + questionType;
      
          selectedQuestions.set(questionKey, {
            id: questionId,               // ✅ FIX: DO NOT put '324-FIB' here
            type: questionType,
            text: questionText,
            module: module
          });
      
          const actionsCell = row.cells[4];
          if (!actionsCell.querySelector('.delete-btn')) {
            actionsCell.innerHTML = `
              <button class="btn btn-danger btn-sm delete-btn">Delete</button>
            `;
          }
        });
      
        updateQuestionIDs();
      }
  
    loadExistingQuestions();
  
    // Add selected questions from modal
    addQuestionButton.addEventListener("click", function (event) {
      event.preventDefault();
  
      checkboxes.forEach((checkbox) => {
        if (checkbox.checked) {
            const questionId = checkbox.dataset.questionId;
            const questionType = checkbox.dataset.questionType;
            const questionKey = questionId + "-" + questionType;
            
            if (!selectedQuestions.has(questionKey)) {
              const row = checkbox.closest("tr");
              const questionText = row.cells[1].innerText;
              const module = row.cells[3].innerText;
            
              selectedQuestions.set(questionKey, {
                id: questionId, // ✅ Fix: do NOT store composite in `id`
                type: questionType,
                text: questionText,
                module: module
              });
            
              const newRow = `
                <tr data-id="${questionId}" data-type="${questionType}">
                  <th scope="row">${questionCounter}</th>
                  <td>${questionText}</td>
                  <td>${questionType}</td>
                  <td>${module}</td>
                  <td>
                    <button class="btn btn-danger btn-sm delete-btn">Delete</button>
                  </td>
                </tr>
              `;
              questionListTable.insertAdjacentHTML("beforeend", newRow);
              questionCounter++;
            }
            
        }
      });
  
      checkboxes.forEach((checkbox) => (checkbox.checked = false));
    });
  
    function handleDelete(questionId, questionType) {
      const key = questionId + "-" + questionType;
      deletedQuestionIds.add(key);
      console.log("Deleted Question IDs after adding:", deletedQuestionIds);
    }
  
    // Delete a question from the table
    questionListTable.addEventListener("click", function (event) {
      if (event.target.classList.contains("delete-btn")) {
        const row = event.target.closest("tr");
        const questionId = row.getAttribute("data-id");
        const questionType = row.getAttribute("data-type");
        const key = questionId + "-" + questionType;
  
        console.log("Retrieved questionId:", questionId);
  
        if (selectedQuestions.has(key)) {
          selectedQuestions.delete(key);
          handleDelete(questionId, questionType);
          row.remove();
          updateQuestionIDs();
  
          console.log("Deleted Question IDs after delete:", deletedQuestionIds);
        } else {
          console.error("Invalid question or not found in selectedQuestions:", questionId);
        }
      }
    });
  
    // Update display IDs
    function updateQuestionIDs() {
      const rows = questionListTable.querySelectorAll("tr");
      let count = 1;
      const updatedMap = new Map();
  
      rows.forEach((row) => {
        const questionId = row.getAttribute("data-id");
        const questionType = row.getAttribute("data-type");
        const questionText = row.cells[1].innerText;
        const module = row.cells[3].innerText;
  
        row.cells[0].innerText = count;
        updatedMap.set(questionId + "-" + questionType, {
          id: questionId,
          type: questionType,
          text: questionText,
          module: module
        });
        count++;
      });
  
      selectedQuestions = updatedMap;
      questionCounter = count;
    }
  
    // Handle form submission
    form.addEventListener("submit", function (event) {
      event.preventDefault();
      if (event.submitter !== addTestButton) return;
  
      console.log("Deleted Question IDs before sending:", deletedQuestionIds);
  
      const filteredQuestions = Array.from(selectedQuestions.values()).filter(
        (q) => !deletedQuestionIds.has(q.id + "-" + q.type)
      ).map(q => ({
        question_id: parseInt(q.id),
        type: q.type
      }));
  
      const testData = {
        test_id: form.getAttribute("data-test-id"),
        name: document.getElementById("new_test")?.value || "",
        cohort: document.querySelector("select[name='cohort']")?.value || "",
        instructions: document.getElementById("skill_content")?.value || "",
        test_type: document.querySelector("select[name='q-type']")?.value || "",
        time_limit: document.getElementById("time_limit")?.value || "",
        module_code: moduleDropdown?.value || "",
        pass_percentage: document.querySelector("input[name='pass_percentage']")?.value || "",
        questions: filteredQuestions,
        questions_to_delete: Array.from(deletedQuestionIds)
      };
  
      console.log("Final testData:", testData);
      console.log("Final questions payload:", filteredQuestions);
      filteredQuestions.forEach(q => {
        console.log("  > ID:", q.question_id, "Type:", q.type, "typeof ID:", typeof q.question_id);
      });
      console.log("🧾 Final questions payload:", testData.questions);
      if (!testData.name || !testData.cohort) {
        alert("Please enter a test name and cohort.");
        return;
      }
      fetch("/save-test", {
        method: "POST",
        headers: { "Content-Type": "application/json" },
        body: JSON.stringify(testData)
      })
        .then(response => {
          if (!response.ok) return response.text().then(text => { throw new Error(text); });
          return response.json();
        })
        .then(data => {
          if (data.error) {
            alert("Error: " + data.error);
          } else if (data.message) {
            alert(data.message);
            form.reset();
            questionListTable.innerHTML = "";
            selectedQuestions.clear();
            questionCounter = 1;
            deletedQuestionIds.clear();
            window.location.href = "/createTests";
          } else {
            alert("Unexpected response from the server.");
          }
        })
        .catch(error => {
          console.error("Fetch Error:", error);
          alert("Network error. Please try again.");
        });
    });
  });
  




